#pragma once

class CHtmlCanvas : private PrintEx
{
private:
	int m_nPageNo;
	int m_nPageCount;
	int m_nMeasureWidth;
	int m_nMeasureHeight;
	int m_nMeasuredHtmlPx;;
	BOOL m_bIsValid;
public:
	CHtmlCanvas(): 
		m_nPageNo(0),
		m_nPageCount(0), 
		m_nMeasureWidth(0), 
		m_nMeasureHeight(0),
		m_bIsValid(false),
		m_nMeasuredHtmlPx(0) {}

	BOOL LoadHtml(const char* url)
	{
		m_nPageNo = 0;
		m_nPageCount = 0;
		m_nMeasureWidth = 0;
		m_nMeasureHeight = 0;
		m_bIsValid = PrintEx::load(url);
		if(m_bIsValid)
		{
			htmlayout::dom::element el = getRootElement();
			htmlayout::dom::element title = el.find_first("title");
			if(title.is_valid())
		        ATLTRACE(TEXT("title: %s\n"), CString(title.text()));
	    }
	    return m_bIsValid;
	}

	BOOL LoadHtml(LPCBYTE data, UINT length)
	{
		m_nPageNo = 0;
		m_nPageCount = 0;
		m_nMeasureWidth = 0;
		m_nMeasureHeight = 0;
		m_bIsValid = PrintEx::load(data, length, "");
		if(m_bIsValid)
		{
			htmlayout::dom::element el = getRootElement();
			htmlayout::dom::element title = el.find_first("title");
			if(title.is_valid())
		        ATLTRACE(TEXT("title: %s\n"), CString(title.text()));
	    }
	    return m_bIsValid;
	}

	BOOL LoadHtmlResource(LPCTSTR pszName, HMODULE hModule = NULL)
	{
		LPBYTE pb;
		DWORD  cb;

		if(!GetHtmlResource(pszName, pb, cb, hModule))
			return FALSE;
		if(strstr((const char*)pb,"@import url(") != NULL)
			return LoadHtmlResourceWithImport(pb, cb);
		return LoadHtml(pb, cb);
	}

	HELEMENT GetRootElement()
	{
		return getRootElement();
	}

	BOOL IncPageNo(int i)
	{
		BOOL bResult = TRUE;
		m_nPageNo += i;
		if(m_nPageNo >= m_nPageCount) 
		{ 
			m_nPageNo = m_nPageCount - 1; 
			bResult = false; 
		}
		if(m_nPageNo < 0)
		{ 
			m_nPageNo = 0; 
			bResult = false; 
		}
		return bResult;
	}

	int GetPageCount()
	{
		return m_nPageCount;
	}

	BOOL Render(HDC hdc, RECT dst)
	{
		if(!m_bIsValid)
			return FALSE;

		int width = dst.right - dst.left;
		int height = dst.bottom - dst.top;

		if(m_nMeasureWidth != width || m_nMeasureHeight != height)
		{
			// measure it
			m_nPageCount = PrintEx::measure(hdc, 816, width, height);

			m_nMeasureWidth = width;
			m_nMeasureHeight = height;
		}

		if(m_nPageNo >= m_nPageCount)
			m_nPageNo = m_nPageCount - 1;
		if(m_nPageNo < 0)
			return FALSE;
		// rendering first page on the metafile
		PrintEx::render(hdc, dst.left, dst.top, m_nPageNo);
		
		return TRUE;
	}

	BOOL Render(HDC hdc, RECT dst, int numOfHTMLPxInPrintingArea)
	{
		if(!m_bIsValid)
			return FALSE;

		int width = dst.right - dst.left;
		int height = dst.bottom - dst.top;

		if(m_nMeasureWidth != width || m_nMeasureHeight != height || m_nMeasuredHtmlPx != numOfHTMLPxInPrintingArea)
		{
			// measure it
			m_nPageCount = PrintEx::measure(hdc, numOfHTMLPxInPrintingArea, width, height);
			m_nMeasureWidth = width;
			m_nMeasureHeight = height;
		}
		if(m_nPageNo >= m_nPageCount)
			m_nPageNo = m_nPageCount - 1;
		if(m_nPageNo < 0)
			return FALSE;
		// rendering first page on the metafile
		PrintEx::render(hdc, dst.left, dst.top, m_nPageNo);
		return TRUE;
	}

	virtual void OnNextPage(unsigned int pageNo, int pageViewportOffsetY, unsigned int& pageViewportHeight)
	{
		htmlayout::dom::element root = getRootElement();
		htmlayout::dom::element body = root.find_first("body");
		RECT rcBody = body.get_location();
		if(unsigned(rcBody.bottom) < (pageViewportOffsetY + pageViewportHeight))
		{  
			// this is the last page.
			int lastPageContent = rcBody.bottom - pageViewportOffsetY;
	    } 
	}
private:
	static inline void AppendBytes(CSimpleValArray<BYTE>& buf, const BYTE* pb, DWORD cb)
	{
		while(cb--)
			buf.Add(*pb++);
	}

	BOOL GetResource(LPCTSTR pszName, LPCTSTR pszType, /*out*/PBYTE& pb, /*out*/DWORD& cb, HMODULE hModule = NULL)
	{
		// Find specified resource and check if ok
		if(!hModule)
			hModule = _Module.GetResourceInstance();
  
		HRSRC hrsrc = ::FindResource(hModule, pszName, pszType);
		if(!hrsrc) 
			return FALSE;

		// Load specified resource and check if ok
		HGLOBAL hgres = ::LoadResource(hModule, hrsrc);
		if(!hgres) 
			return FALSE;

		// Retrieve resource data and check if ok
		pb = (PBYTE)::LockResource(hgres); if (!pb) return FALSE;
		cb = ::SizeofResource(hModule, hrsrc); if (!cb) return FALSE;
		
		return TRUE;
	}

	BOOL LoadHtmlResourceWithImport(const BYTE* pb, DWORD cb)
	{
		CSimpleValArray<BYTE> buf;
		
		const BYTE *pend = pb + cb;
		while(pb < pend)
		{
			const BYTE* insPoint = (const BYTE*) strstr((const char*)pb,"@import url(");
			if(insPoint == NULL) 
			{
				// no more @import found...
				AppendBytes( buf, pb, pend - pb );
				break;
			}
			else
			{
				AppendBytes(buf, pb, insPoint - pb);
				insPoint += 12; //sizeof("@import url(")
				const BYTE *tail = (const BYTE*) strstr( (const char*) insPoint, ")");
				ATLASSERT(tail);
				CString resName((const char *)insPoint, tail - insPoint);
				PBYTE pbh;
				DWORD cbh;
				const BYTE* point = (const BYTE*) strstr((const char*)insPoint, ".");
				ATLASSERT(point);
				CString resBaseName((const char *)insPoint, point - insPoint);
				CString resType((const char *)point+1, tail - point - 1);
				if(GetResource(resBaseName, resType, pbh, cbh))
					AppendBytes(buf, pbh, cbh);
				else
					buf.Add('?');
				pb = tail + 1; //")"
			}
		}
		return LoadHtml(buf.GetData(),  buf.GetSize());
  }

};
