// htmlview.h: interface for the CHtmlView class.
//
//////////////////////////////////////////////////////////////////////
#pragma once

#include "htmlayout_aux.h"
#include "htmlcanvas.h"

class CHtmlView : 
	public CWindowImpl<CHtmlView, CHTMLayoutCtrl>,
	public CHTMLayoutHost<CHtmlView>,
	public htmlayout::event_handler
{
	HELEMENT  m_mouseOver;
public:
	CHtmlView() : event_handler( unsigned(-1) /*handle all event groups*/ ) {}

	BOOL PreTranslateMessage(MSG* pMsg)
	{
		// HTMEngine needs tabs for TAB navigation

		if((pMsg->message == WM_KEYDOWN || pMsg->message == WM_KEYUP) && ::GetFocus() == m_hWnd)
		{
			if(pMsg->wParam == VK_TAB) // always need this
			{
				::SendMessage(m_hWnd,pMsg->message,pMsg->wParam,pMsg->lParam);
				return TRUE;
			}
		}
		return FALSE;
	}

	BEGIN_MSG_MAP(CHtmlView)
		MESSAGE_HANDLER(WM_CREATE, OnCreate)
		MESSAGE_HANDLER(WM_MOUSEMOVE, OnMouseMove)
		MESSAGE_HANDLER(WM_RBUTTONUP, OnMouseUp)
		MESSAGE_HANDLER(WM_DROPFILES, OnDropFiles)
		HTML_HYPERLINK_HANDLER(OnHyperlink)
	END_MSG_MAP()

	LRESULT OnCreate(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
	{
		DefWindowProc();
		// HTMLayout stuff
		SetCallback(); // reroute all notifications here
		SetEventHandler(this); // set this as a "zero level" event handler, all not handled HTML DOM events will go here
		LoadHtmlResource(_T("TABLE"));
		::DragAcceptFiles(m_hWnd, TRUE);
		return 0;
	}

	LRESULT OnHyperlink(NMHL_HYPERLINK* pnmhl, BOOL& bHandled)
	{
		ATLTRACE(_T("CHTMLayoutHost::OnHyperLink: href='%.256s' target='%s' action=%u\n"), 
		CString(pnmhl->szHREF), CString(pnmhl->szTarget), pnmhl->action);

		switch (pnmhl->action) 
		{
			case NMHL_HYPERLINK::ENTER: 
				break;

			case NMHL_HYPERLINK::LEAVE: 
				break;

			case NMHL_HYPERLINK::CLICK: 
				break;
		}
		return 0;
	}

	HELEMENT in_focus()
	{
		return htmlayout::dom::element::focus_element(m_hWnd);
	}

	virtual BOOL handle_focus(HELEMENT he, FOCUS_PARAMS& params ) 
	{
		return FALSE;
	}

	virtual BOOL handle_event (HELEMENT he, BEHAVIOR_EVENT_PARAMS& params ) 
	{ 
		htmlayout::dom::element t = params.heTarget;
		CString id = t.get_attribute("id");
		CString val = t.text();
		htmlayout::dom::element root = htmlayout::dom::element::root_element(m_hWnd);

		switch(params.cmd)
        {
			case EDIT_VALUE_CHANGING:
				break;

			case EDIT_VALUE_CHANGED:
				break;

			case BUTTON_STATE_CHANGED:
				break;

			case BUTTON_CLICK:
				if(id == _T("test1"))
					DoTest1();
				if(id == _T("test2"))
					DoTest2();
				if(id == _T("test3"))
					DoTest3();
				break;

			case POPUP_DISMISSED:
				break;

			case MENU_ITEM_CLICK:
				break;
		}
		return FALSE; 
	}

	LRESULT OnMouseMove(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		bHandled = FALSE;
		return 0;
	}

	static void CALLBACK styleCallback( HTMLayoutCSSRuleDef* pdef, LPVOID callback_prm )
	{
	}

	LRESULT OnMouseUp(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		bHandled = FALSE;
		return 0;
	}

	LRESULT OnDropFiles(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		HDROP hdrop = (HDROP) wParam; 

		DragFinish ( hdrop );
		return 0;
	}

	void DoTest1()
	{
		htmlayout::dom::element root = htmlayout::dom::element::root_element(m_hWnd);
		htmlayout::dom::element table = root.find_first("table#table1");
		htmlayout::dom::element tr = table.find_first("tr#table_f");
		
		json::astring html_before = tr.get_html();

		tr.clear();
		htmlayout::dom::element th1 = htmlayout::dom::element::create("th", L"AA");
		htmlayout::dom::element th2 = htmlayout::dom::element::create("th", L"BB");
		htmlayout::dom::element th3 = htmlayout::dom::element::create("th", L"CC");
		tr.append(th1);
		tr.append(th2);
		tr.append(th3);
		table.update(true);

		json::astring html_after = tr.get_html();
	}

	void DoTest2()
	{
		htmlayout::dom::element root = htmlayout::dom::element::root_element(m_hWnd);
		htmlayout::dom::element table = root.find_first("table#table1");
		htmlayout::dom::element tr = table.find_first("tr#replace");

		for(int i = (int)tr.children_count(); i > 0; i--)
		{
			htmlayout::dom::element t = tr.child(i-1);
			CString s = t.get_attribute("mark");
			if(!s.IsEmpty() && s == _T("replace"))
			{
				htmlayout::dom::element t = tr.child(i-1);
				t.detach();
				//t.destroy();
			}
		}
		json::astring html_after1 = tr.get_html();

		htmlayout::dom::element td1 = htmlayout::dom::element::create("td", L"AA");
		htmlayout::dom::element td2 = htmlayout::dom::element::create("td", L"BB");
		tr.append(td1);
		tr.append(td2);

		table.update(true);
		json::astring html_after2 = tr.get_html();
	}

	void DoTest3()
	{
		htmlayout::dom::element root = htmlayout::dom::element::root_element(m_hWnd);
		htmlayout::dom::element body = root.find_first("body");
		htmlayout::dom::element table1 = root.find_first("table#table1");
		htmlayout::dom::element table2 = root.find_first("table#table2");
		htmlayout::dom::element tc1 = table1.clone();
		htmlayout::dom::element tc2 = table2.clone();
		tc1.set_attribute("id", L"");
		tc2.set_attribute("id", L"");
		body.append(tc1);
		body.append(tc2);
		root.update(true);
		
		json::astring html_after = tc2.get_html();
	}
};
